@@ -76,7 +76,7 @@
 	({								       \
 		MAYBE_BUILD_BUG_ON((alignment) == 0			       \
 				   || ((alignment) & ((alignment) - 1)));      \
-		(((align_drift) - (alignment)) & ((alignment) - 1);	       \
+		(((align_drift) - (alignment)) & ((alignment) - 1));	       \
 	})
 
 #endif /* _BABELTRACE_ALIGN_H */
